import java.util.Stack;

/**
 * @author Hamid Ebadi
 *
 */
public class Calculator {
	// operators 
	private enum CalcOperator {
		PLUS, MINUS , MULTIPLY , DIVISION , EQUAL, BRACKET, UNDEFINED;
	}

	// Calculator State 
	private final Stack<CalcOperator> mOperatorStack = new Stack<CalcOperator>();
	private final Stack<Integer> mOperandStack = new Stack<Integer>();
	private Boolean CSValidBuffer ;   
	private String buffer  ;
	private Boolean InBracket ;
	private Calculator BracketCalc ;
	private Boolean error= false ;
	private String memory;
	// Methods
	// Constructor 
	Calculator (){
		pressReset();
	}
	
	// ~~~ Reset ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void pressReset(){
		error = false ;
		buffer = "" ;
		mOperatorStack.clear();
		mOperatorStack.push(CalcOperator.UNDEFINED);
		mOperandStack.clear();
		mOperandStack.push(0);
		CSValidBuffer = false;
		InBracket= false;
		BracketCalc= null;
		memory = null;
	}
	
	// ~~~ Digits ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void pressDigit(char digit){
		if (error) return;
			if (!Character.isDigit(digit)){
				System.out.println("Invalid digit" );
			}else {		  
				if (CSValidBuffer){
					buffer  = buffer + digit ;
				}else {
					buffer = String.valueOf(digit);
					CSValidBuffer = true ;
				}
			}
	}
	
	// ~~~ Operators ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void pressPlus(){
		pressOp(CalcOperator.PLUS);
	}
	public void pressMinus(){
		pressOp(CalcOperator.MINUS);
	}
	public void pressMult(){
		pressOp(CalcOperator.MULTIPLY);
	}
	public void pressDiv(){
		pressOp(CalcOperator.DIVISION);
	}
	
	private void pressOp(CalcOperator op) {
		if (error) return;
		this.calculate();
		this.setOperand( this.BufferToInteger() );
		this.setOperator(op) ;
		CSValidBuffer = false ;
	}
	// ~~~ Memory operations ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void pressM() {
		if (error) return;
		this.memory = this.buffer;
	}

	public void pressMR() {
		if (error) return;
		if (memory == null)
			this.error();
		else {
			this.buffer = this.memory;
			CSValidBuffer = false;
		}
	}
	
	public void pressMC() {
		if (error) return;
		this.memory = "";
	}

	private void error() {
		this.error = true;
		this.buffer = "Err";
	}
	
	public void pressOpenBracket(){
		if (error) return;
		mOperatorStack.push(CalcOperator.UNDEFINED);
		mOperandStack.push(0);
	}
	
	public void pressCloseBracket(){
		if (error) return;
		this.calculate();
		mOperatorStack.pop();
		mOperandStack.pop();
	}

	public void pressEqual(){
		if (error) return ;
		this.calculate();
	}
	
	/* Perform calculation and replace the buffer with the result */
	private void calculate () { 
		CalcOperator operator = this.getOperator();
		Integer operand = this.getOperand();
		int result = 0;
		switch (operator) {
		case PLUS:
			result = operand + BufferToInteger();
			break;
		case MINUS:
			result = operand + BufferToInteger();
			break;
		case MULTIPLY:
			result = operand * BufferToInteger();
			break;
		case DIVISION:
			result = operand / BufferToInteger();
			break;
		default:
			result = BufferToInteger();
		}
		this.buffer = Integer.toString(result);
	}
	public String getBuffer(){
		return buffer;	
	}

	private Integer BufferToInteger(){
		if (buffer.length()<4){
			return Integer.valueOf(buffer);
		}else {
			System.out.println("too long digit");
			return 0; 
		}
	}
	
	// ~~~ State manipulation ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private Integer getOperand() {
		return mOperandStack.peek();
	}
	
	private void setOperand(Integer i) {
		mOperandStack.pop();
		mOperandStack.push(i);
	}
	
	private CalcOperator getOperator() {
		return mOperatorStack.peek();
	}
	
	private void setOperator(CalcOperator o) {
		mOperatorStack.pop();
		mOperatorStack.push(o);
	}
	
	public static void main(String[] args) {
		Calculator calc= new Calculator();
		if (args.length < 1) System.out.println("Missing command-line arguments. ( e.g. 1+3 -4= )\n " );
		for (int i = 0 ; i!= args.length ; i++) {
			for (int j = 0 ; j!= args[i].length() ; j++) {
				char ch = args[i].charAt(j) ;
				if (Character.isDigit(ch)){
					calc.pressDigit(ch);
				}
				switch(ch){
				case '+' :
					calc.pressPlus();
					break;
				case '-' :
					calc.pressMinus();
					break;
				case '/':
					calc.pressDiv();
					break;
				case '*':
					calc.pressMult();
					break;
				case '=':
					calc.pressEqual();
					break;
				case '(':
					calc.pressOpenBracket();
					break;
				case ')':
					calc.pressCloseBracket();
					break;	
				default :
					
				}
				System.out.println("Result of proccessing \"" + ch + "\" : Operand(Memory):" + calc.getOperand()+ ", Digit buffer:" + calc.getBuffer());
			}
		}

	}

}